==> vradi-services-0.0.4-sources.jar: com/jurismarches/vradi/services/search/VradiQueryParser.java <==
/*
 * *##%
 * Vradi :: Services
 * Copyright (C) 2009 - 2010 JurisMarches, Codelutin
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Lesser Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/gpl-3.0.html>.
 * ##%*
 */
package com.jurismarches.vradi.services.search;

import java.util.List;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.lucene.analysis.Analyzer;
import org.apache.lucene.analysis.WhitespaceAnalyzer;
import org.apache.lucene.queryParser.ParseException;
import org.apache.lucene.queryParser.QueryParser;
import org.apache.lucene.search.Query;
import org.sharengo.wikitty.search.Search;

/**
 * VradiQueryParser supports the analysis of textual queries.
 * 
 * Queries are analyzed by the lucene <code>QueryParser</code>, so they
 * must obey his syntax.
 * See at <a href="http://lucene.apache.org/java/2_4_0/queryparsersyntax.html">
 * queryparsersyntax.html</a> for a complete documentation.
 * 
 * Analyzer used to parse queries is <code>WhitespaceAnalyzer</code> and the default
 * parser operator is <code>QueryParser.Operator.AND</code>.
 * 
 * @author $Author: schorlet $
 * @version $Revision: 473 $ $Date: 2010-02-06 00:31:12 +0100 (sam., 06 févr. 2010) $
 * @since 0.0.4
 */
public class VradiQueryParser {
    private static final Log log = LogFactory.getLog(VradiQueryParser.class);
    public static final String DEFAULT_FIELD = "text";
    
    /**
     * Parses the specified <code>expression</code> and returns a <code>FilterList</code>.
     * 
     * @param expression the expression
     * @return a <code>FilterList</code>
     * @throws UnsupportedQueryException
     */
    public static FilterList parse(String expression) throws UnsupportedQueryException {
        return parse(expression, DEFAULT_FIELD);
    }
    
    /**
     * Parses the specified <code>expression</code> and returns a <code>FilterList</code>.
     * 
     * @param expression the expression
     * @param defaultField the default search field
     * @return a <code>FilterList</code>
     * @throws UnsupportedQueryException
     */
    public static FilterList parse(String expression, String defaultField) throws UnsupportedQueryException {
        FilterListQueryHandler queryHandler = new FilterListQueryHandler();
        parse(queryHandler, expression, defaultField);
        
        FilterList filterList = queryHandler.getFilterList();
        return filterList;
    }
    
    static void parse(QueryHandler queryHandler, String expression, String defaultField) throws UnsupportedQueryException {
        if (log.isDebugEnabled()) {
            log.debug("expression: " + expression);
        }
        
        Analyzer analyzer = new WhitespaceAnalyzer();
        QueryParser parser = new QueryParser(defaultField, analyzer);
        
        parser.setAllowLeadingWildcard(true);
        parser.setDefaultOperator(QueryParser.Operator.AND);
        
        if (expression == null || expression.isEmpty()) {
            expression = defaultField + ":*";
        }
        
        try {
            Query query = parser.parse(expression);
            
            LuceneQueryParser luceneQueryParser = new LuceneQueryParser(queryHandler);
            luceneQueryParser.parse(query);
            
        } catch (ParseException e) {
            throw new UnsupportedQueryException(expression, e);
        }
    }
    
    /**
     * Parses the specified <code>expression</code> and invokes
     * {@link #buildSearch(FilterList, Search)} with a new <code>Search</code> instance.
     * 
     * @param expression the expression
     * @return a <code>Search</code>
     * @throws UnsupportedQueryException
     */
    public static Search search(String expression) throws UnsupportedQueryException {
        FilterList filterList = VradiQueryParser.parse(expression);
        
        Search search = Search.query();
        buildSearch(filterList, search);

        return search;
    }
    
    /**
     * Appends Filters from the specified <code>list</code>
     * to the specified <code>search</code>.
     * 
     * @param list the list of filters
     * @param search the search
     */
    public static void buildSearch(FilterList list, Search search) {
        Search subSearch = null;
        FilterList.Operator operator = list.getOperator();

        if (operator == FilterList.Operator.MUST_PASS_ONE) {
            subSearch = search.or();
            
        } else if (operator == FilterList.Operator.MUST_PASS_ALL) {
            subSearch = search.and();
            
        } else if (operator == FilterList.Operator.MUST_NOT_PASS) {
            subSearch = search.not();
        }
        
        List<Filter> filters = list.getFilters();
        for (Filter filter : filters) {
            
            if (filter instanceof FilterList) {
                buildSearch((FilterList) filter, subSearch);
        
            } else if (filter instanceof CompareFilter) {
                CompareFilter compareFilter = (CompareFilter) filter;
                
                String name = compareFilter.getName();
                String value = compareFilter.getValue();
                
                subSearch.eq(name, value);
            }
        }
    }
}

==> vradi-services-0.0.5-sources.jar: com/jurismarches/vradi/services/search/VradiQueryParser.java <==
/*
 * *##%
 * Vradi :: Services
 * Copyright (C) 2009 - 2010 JurisMarches, Codelutin
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Lesser Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/gpl-3.0.html>.
 * ##%*
 */
package com.jurismarches.vradi.services.search;

import java.util.List;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.lucene.analysis.Analyzer;
import org.apache.lucene.analysis.WhitespaceAnalyzer;
import org.apache.lucene.queryParser.ParseException;
import org.apache.lucene.queryParser.QueryParser;
import org.apache.lucene.search.Query;
import org.apache.lucene.util.Version;
import org.sharengo.wikitty.search.Search;

/**
 * VradiQueryParser supports the analysis of textual queries.
 * 
 * Queries are analyzed by the lucene <code>QueryParser</code>, so they
 * must obey his syntax.
 * See at <a href="http://lucene.apache.org/java/2_9_0/queryparsersyntax.html">
 * queryparsersyntax.html</a> for a complete documentation.
 * 
 * Analyzer used to parse queries is <code>WhitespaceAnalyzer</code> and the default
 * parser operator is <code>QueryParser.Operator.AND</code>.
 * 
 * @author $Author: schorlet $
 * @version $Revision: 556 $ $Date: 2010-02-24 22:36:49 +0100 (mer., 24 févr. 2010) $
 * @since 0.0.4
 */
public class VradiQueryParser {
    private static final Log log = LogFactory.getLog(VradiQueryParser.class);
    public static final String DEFAULT_FIELD = "text";
    
    /**
     * Parses the specified <code>expression</code> and returns a <code>FilterList</code>.
     * 
     * @param expression the expression
     * @return a <code>FilterList</code>
     * @throws UnsupportedQueryException
     */
    public static FilterList parse(String expression) throws UnsupportedQueryException {
        return parse(expression, DEFAULT_FIELD);
    }
    
    /**
     * Parses the specified <code>expression</code> and returns a <code>FilterList</code>.
     * 
     * @param expression the expression
     * @param defaultField the default search field
     * @return a <code>FilterList</code>
     * @throws UnsupportedQueryException
     */
    public static FilterList parse(String expression, String defaultField) throws UnsupportedQueryException {
        FilterListQueryHandler queryHandler = new FilterListQueryHandler();
        parse(queryHandler, expression, defaultField);
        
        FilterList filterList = queryHandler.getFilterList();
        return filterList;
    }
    
    static void parse(QueryHandler queryHandler, String expression, String defaultField) throws UnsupportedQueryException {
        if (log.isDebugEnabled()) {
            log.debug("expression: " + expression);
        }
        
        Analyzer analyzer = new WhitespaceAnalyzer();
        QueryParser parser = new QueryParser(Version.LUCENE_29, defaultField, analyzer);
        
        parser.setAllowLeadingWildcard(true);
        parser.setDefaultOperator(QueryParser.Operator.AND);
        
        if (expression == null || expression.trim().isEmpty()) {
            expression = defaultField + ":*";
        }

        try {
            Query query = parser.parse(expression.trim());
            
            LuceneQueryParser luceneQueryParser = new LuceneQueryParser(queryHandler);
            luceneQueryParser.parse(query);
            
        } catch (ParseException e) {
            throw new UnsupportedQueryException(expression, e);
        }
    }
    
    /**
     * Parses the specified <code>expression</code> and invokes
     * {@link #buildSearch(FilterList, Search)} with a new <code>Search</code> instance.
     * 
     * @param expression the expression
     * @return a <code>Search</code>
     * @throws UnsupportedQueryException
     */
    public static Search search(String expression) throws UnsupportedQueryException {
        FilterList filterList = VradiQueryParser.parse(expression);
        
        Search search = Search.query();
        buildSearch(filterList, search);

        return search;
    }
    
    /**
     * Appends Filters from the specified <code>list</code>
     * to the specified <code>search</code>.
     * 
     * @param list the list of filters
     * @param search the search
     */
    public static void buildSearch(FilterList list, Search search) {
        Search subSearch = null;
        FilterList.Operator operator = list.getOperator();

        if (operator == FilterList.Operator.MUST_PASS_ONE) {
            subSearch = search.or();
            
        } else if (operator == FilterList.Operator.MUST_PASS_ALL) {
            subSearch = search.and();
            
        } else if (operator == FilterList.Operator.MUST_NOT_PASS) {
            subSearch = search.not();
        }
        
        List<Filter> filters = list.getFilters();
        for (Filter filter : filters) {
            
            if (filter instanceof FilterList) {
                buildSearch((FilterList) filter, subSearch);
        
            } else if (filter instanceof CompareFilter) {
                CompareFilter compareFilter = (CompareFilter) filter;
                
                String name = compareFilter.getName();
                String value = compareFilter.getValue();
                
                subSearch.eq(name, value);
            }
        }
    }
}
